///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_ARGVPARSER_H
#define CHUSTD_ARGVPARSER_H

#include "String.h"
#include "Array.h"

namespace chustd {

// Allows to parse arguments from the command line
class ArgvParser  
{
public:
	ArgvParser(const String& strCmdLine);

	// Returns true if a flag word is present in the command line
	// Example: compil -Verbose // GetFlag("verbose") returns true
	bool GetFlag(const String& strFlagName) const;

	// Returns the value assigned to a flag
	// Example: run -value:"100" // GetFlagString("value") returns "100"
	String GetFlagString(const String& strFlagName) const;
	
	virtual ~ArgvParser();

protected:
	String m_strCmdLine;

	class Arg
	{
	public:
		bool m_bFlag;
		String m_strName;
		String m_strFlagString;
	};
	Array<Arg> m_aArgs;

private:
	void Parse();
	void FirstPassSplit(const wchar* psz, int length, StringArray& astrArgs);
	void ConvertToArgs(const StringArray& astrArgs);
	int32 GetFlagIndex(const String& strFlagName) const;
	const wchar* JumpOverExeName(const wchar* pszCmdLine);
};

}

#endif
