///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Directory.h"
#include "String.h"
#include "Array.h"
#include "File.h"

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////

Directory::Directory()
{

}

Directory::~Directory()
{

}

// [in] fullPath : true to get paths as dir+name instead of name only
StringArray Directory::GetFileNames(const String& strDir, const String& strJoker, bool fullPaths)
{
	StringArray filePaths;
	StringArray jokers = strJoker.Split('|');
	if( jokers.IsEmpty() )
	{
		jokers.Add("*.*");
	}
	foreach(jokers, iJoker)
	{
		String pathToExplore = FilePath::Combine(strDir, jokers[iJoker]);

#ifdef WIN32
		WIN32_FIND_DATAW fdw;

		HANDLE hFind = FindFirstFileW(pathToExplore.GetBuffer(), &fdw);
		if( hFind != INVALID_HANDLE_VALUE )
		{
			BOOL bFind = TRUE;
			do
			{
				// We do not want the "." nor the ".." directories

				bool bDot = (fdw.cFileName[0] == '.' && fdw.cFileName[1] == 0);
				bool bDotDot = (fdw.cFileName[0] == '.' && fdw.cFileName[1] == '.' && fdw.cFileName[2] == 0);

				if( !bDot && !bDotDot )
				{
					String strFileName = fdw.cFileName;
					if( fullPaths )
					{
						filePaths.Add( FilePath::Combine(strDir, strFileName) );
					}
					else
					{
						filePaths.Add(strFileName);
					}
				}

				bFind = FindNextFileW(hFind, &fdw);
			}while(bFind);
		}
		FindClose(hFind);
#elif CHUOS
#endif
	}
	filePaths.SetAsReturnValue();
	return filePaths;
}

bool Directory::Create(const String& strDir)
{
#ifdef WIN32

	BOOL bOk = ::CreateDirectoryW(strDir.GetBuffer(), NULL);
	return bOk != FALSE;

#elif CHUOS
	IFileSystem* pFileSystem = null;
	String strParentDir, strSubDir;
	File::SplitPathName(strDir, strParentDir, strSubDir);
	return pFileSystem->CreateDirectory(strParentDir, strSubDir);

#else
	// TODO
	ASSERT(false);
	return false;
	
#endif
}

bool Directory::Exists(const String& strDir)
{
	bool bExists = false;

#ifdef WIN32
	String strDir2 = FilePath::AddSeparator(strDir, '\\') + L".";
	
	DWORD attributes = GetFileAttributes(strDir2.GetBuffer());
	if( attributes != INVALID_FILE_ATTRIBUTES )
	{
		bExists = (attributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
	}
#elif CHUOS
#endif
	return bExists;
}

bool Directory::Delete(const String& strDir)
{

#ifdef WIN32
	BOOL bOk = RemoveDirectoryW(strDir.GetBuffer());
	return bOk != FALSE;
#elif CHUOS
	return false;
#else
	// TODO
	ASSERT(false);
	return false;
#endif
}
