///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Dns.h"
#include "Socket.h"
#include "TextEncoding.h"

using namespace chustd;

bool Dns::GetHostByName(const chustd::String& strName, HostEntry& he)
{
#ifdef WIN32
	// Force socket api startup
	Socket sock;

	// Note: pServInfo->h_addr == pServInfo->h_addr_list[0]
	ByteArray aNameAscii = strName.ToBytes( TextEncoding::Iso8859_1(), true);
	struct hostent* pOldStruct = gethostbyname((const char*) aNameAscii.GetPtr());
	if( pOldStruct == null )
		return false;

	// Conversion from sz char* to String
	he.m_strName = String::FromAsciiSZ(pOldStruct->h_name);

	// Fill the aliases array
	for(int iAlias = 0; iAlias < 128; ++iAlias)
	{
		const char* pszAlias = pOldStruct->h_aliases[iAlias];
		if( pszAlias == null )
			break;
		he.m_astrAliases.Add( String::FromAsciiSZ(pszAlias));
	}

	// Fill the ip array
	const uint8** papAddresses = (const uint8**) pOldStruct->h_addr_list;
	for(int iAddress = 0; iAddress < 128; ++iAddress)
	{
		const uint8* pAddress = papAddresses[iAddress];
		if( pAddress == null )
			break;

		uint8 nA = pAddress[0];
		uint8 nB = pAddress[1];
		uint8 nC = pAddress[2];
		uint8 nD = pAddress[3];

		uint32 nIpAddress = (nA << 24) | (nB << 16) | (nC << 8) | nD;
		he.m_aIpAddresses.Add(nIpAddress);
	}
	return true;

#elif CHUOS

	return false;
#endif
}