///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Process.h"
#include "String.h"
#include "Array.h"
#include "File.h"

using namespace chustd;

String Process::GetCurrentDirectory()
{
#ifdef WIN32
	// Includes ending 0
	const int sizeofBuffer = ::GetCurrentDirectory(0, NULL);

	wchar* pBuffer = new wchar[sizeofBuffer];
	::GetCurrentDirectory(sizeofBuffer, pBuffer);

	String strDirectory(pBuffer);

	delete[] pBuffer;
	return strDirectory;

#elif CHUOS
	return String();
#endif
}

void Process::SetCurrentDirectory(const String& strDirectory)
{
#ifdef WIN32
	::SetCurrentDirectory(strDirectory.GetBuffer());
#elif CHUOS
	
#endif
}

uint16 Process::GetCurrentId()
{
#ifdef WIN32
	uint32 nId = ::GetCurrentProcessId();
	return uint16(nId);
#elif CHUOS
	return 0;	
#endif
}

String Process::GetCommandLine()
{
#ifdef WIN32
	LPWSTR psz = ::GetCommandLineW();
	return String(psz);
#elif CHUOS
	return String();	
#endif
}

chustd::StringArray Process::CommandLineToArgv(const String& strCmdLine)
{
	StringArray astrResult;
	astrResult.SetAsReturnValue();
	
	enum { stateInsideArg, stateInsideQuotedArg, stateCalm};
	
	int state = stateCalm;
	int32 argStart = -1;

	const int32 length = strCmdLine.GetLength();
	for(int iChar = 0; iChar < length; ++iChar)
	{
		wchar c = strCmdLine.GetAt(iChar);
		if( state == stateCalm )
		{
			if( c == L'"' )
			{
				state = stateInsideQuotedArg;
				argStart = iChar + 1;
			}
			else if( c != L' ' )
			{
				state = stateInsideArg;
				argStart = iChar;
			}
		}
		else if( state == stateInsideQuotedArg )
		{
			if( c == L'"' )
			{
				// End
				ASSERT(argStart >= 0);
				String strArg = strCmdLine.Mid(argStart, iChar - argStart);
				astrResult.Add(strArg);
				state = stateCalm;
			}
		}
		else if( state == stateInsideArg )
		{
			if( c == L' ' )
			{
				ASSERT(argStart >= 0);
				String strArg = strCmdLine.Mid(argStart, iChar - argStart);
				astrResult.Add(strArg);
				state = stateCalm;
			}
		}
	}

	// Last argument
	if( state == stateInsideArg )
	{
		ASSERT(argStart >= 0);
		String strArg = strCmdLine.Mid(argStart);
		astrResult.Add(strArg);
		state = stateCalm;
	}

	return astrResult;
}

String Process::GetExecutablePath()
{
	StringArray astrArgs = Process::CommandLineToArgv( GetCommandLine());
	if( astrArgs.GetSize() == 0 )
	{
		return String();
	}
	return astrArgs[0];
}

String Process::GetExecutableDirectory()
{
	String strPath = GetExecutablePath();
	
	String strDir, strFileName;
	FilePath::Split(strPath, strDir, strFileName);
	return strDir;
}
