///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Semaphore.h"

using namespace chustd;

Semaphore::Semaphore()
{
	m_handle = 0;
}

Semaphore::~Semaphore()
{
	Close();
}

bool Semaphore::Create(int initialCount)
{
	m_handle = CreateSemaphore(NULL, initialCount, MAX_INT32, NULL);
	return m_handle != NULL;
}

void Semaphore::Close()
{
	if( m_handle )
	{
		CloseHandle(m_handle);
		m_handle = 0;
	}
}

bool Semaphore::Increment(int count)
{
	LONG previousCount = 0;
	return ReleaseSemaphore(m_handle, count, &previousCount) != 0;
}

bool Semaphore::Decrement()
{
	LONG previousCount = 0;
	return ReleaseSemaphore(m_handle, -1, &previousCount) != 0;
}

int Semaphore::Wait(int timeout)
{
	DWORD wret = WaitForSingleObject(m_handle, timeout);
	if( wret == WAIT_OBJECT_0 )
	{
		return 0;
	}
	else if( wret == WAIT_TIMEOUT )
	{
		return -1;
	}
	else
	{
		return -2;
	}
}
