///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Thread.h"

using namespace chustd;

////////////////////////////////////////////////
Thread::Thread()
{
	m_handle = 0;
}

////////////////////////////////////////////////
Thread::~Thread()
{
	Close();
}

////////////////////////////////////////////////
struct StartArg
{
	int (*pfn)(void*);
	void* userArg;
};

////////////////////////////////////////////////
static DWORD WINAPI ThreadProc(LPVOID pArg)
{
	StartArg startArg = *(StartArg*)pArg;
	delete (StartArg*)pArg;

	return startArg.pfn(startArg.userArg);
}

////////////////////////////////////////////////
bool Thread::Start(int (*pfn)(void*), void* userArg)
{
	DWORD threadId = 0;
	StartArg* pStartArg = new StartArg;
	pStartArg->pfn = pfn;
	pStartArg->userArg = userArg;
	m_handle = ::CreateThread(NULL, 0, ThreadProc, pStartArg, 0, &threadId);
	return m_handle != NULL;
}

////////////////////////////////////////////////
void Thread::Close()
{
	if( m_handle )
	{
		CloseHandle(m_handle);
		m_handle = 0;
	}
}

////////////////////////////////////////////////
bool Thread::IsStarted() const
{
	return m_handle != 0;
}

////////////////////////////////////////////////
void Thread::Suspend()
{
	::SuspendThread(m_handle);
}

////////////////////////////////////////////////
void Thread::WaitForExit()
{
	if( m_handle )
	{
		WaitForSingleObject(m_handle, INFINITE);
	}
}

////////////////////////////////////////////////
void Thread::Sleep(int delayMs)
{
#ifdef WIN32
	::Sleep(delayMs);
#elif CHUOS
	
#endif
}

