///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2013 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UdpSocket.h"
#include "Memory.h"

using namespace chustd;

//////////////////////////////////////////////////////////////////////

int32 UdpSocket::SendTo(void* pBuffer, int32 bufferSize, int32 flags, uint32 nIpAddress, uint16 port)
{
	if( !IsOpen() )
	{
		if( !super::Open(typeUdp) )
			return -1;
	}

#ifdef WIN32
	struct sockaddr_in saiTo;
	const int32 saiSize = sizeof(saiTo);

	Memory::Zero(&saiTo, saiSize);

	saiTo.sin_addr.s_addr = htonl(nIpAddress);
	saiTo.sin_family = AF_INET;
	saiTo.sin_port = htons(port);
	
	const int64 socketId = GetSocketID();
	return sendto( SOCKET_ID(socketId), (const char*) pBuffer, bufferSize, flags, (struct sockaddr*) &saiTo, saiSize);
#elif CHUOS
	return 0;
#endif
}

int32 UdpSocket::ReceiveFrom(void* pBuffer, int32 bufferSize, int32 flags, uint32& nIpAddress, uint16& port)
{
	if( !IsOpen() )
	{
		if( !super::Open(typeUdp) )
			return -1;
	}

#ifdef WIN32
	struct sockaddr_in saiFrom;
	const int32 saiSize = sizeof(saiFrom);

	Memory::Zero(&saiFrom, saiSize);

	const int64 socketId = GetSocketID();
	int fromSize = saiSize;
	int32 read = recvfrom( SOCKET_ID(socketId), (char*) pBuffer, bufferSize, flags, (struct sockaddr*) &saiFrom, &fromSize);

	nIpAddress = ntohl(saiFrom.sin_addr.s_addr);
	port = ntohs(saiFrom.sin_port);
	return read;
#elif CHUOS
	return 0;
#endif
}
